import dataclasses
import datetime
import typing

from mpdab.exceptions import InvalidItemException

StickerMap = typing.Dict[str, str]


@dataclasses.dataclass(order=True)
class Item:
    uri: str = dataclasses.field(compare=False)
    audiobook_name: str  # type: ignore
    sequence: int  # type: ignore
    last_position: typing.Optional[float] = dataclasses.field(
        default=None, compare=False)
    last_played: typing.Optional[datetime.datetime] = dataclasses.field(
        default=None, compare=False)


def parse_stickers(stickers: typing.List[str]) -> StickerMap:
    sticker_map = {}
    for sticker in stickers:
        k, v = sticker.split('=', 1)
        sticker_map[k] = v
    return sticker_map


def get_item_from_sticker_map(song_uri: str, stickers: StickerMap) -> Item:
    if not all((
        stickers,
        stickers.get('audiobook'),
        stickers.get('sequence'),
    )):
        raise InvalidItemException(
            'Item cannot be instanciated from provided stickers.')

    return Item(
        song_uri,
        stickers['audiobook'],
        int(stickers['sequence']),
        float(stickers['last_position'])
        if 'last_position' in stickers else None,
        datetime.datetime.fromisoformat(stickers['last_played'])
        if 'last_played' in stickers else None,
    )
